@@ -2,9 +2,6 @@
 Register processes, updaters, dividers, serializers upon import
 """
 
-# import matplotlib to help fix bug with import order
-import matplotlib.pyplot as plt
-
 # import registries
 from vivarium.core.registry import (
     process_registry,
@@ -44,10 +41,9 @@ from vivarium.core.registry import (
     divide_null, divide_binomial, divide_set_value,
 )
 from vivarium.core.serialize import (
-    ProcessSerializer, NumpySerializer, SequenceDeserializer,
-    NumpyBoolSerializer, UnitsSerializer, DictDeserializer,
-    FunctionSerializer, NumpyInt64Serializer, NumpyInt32Serializer,
-    NumpyFloat32Serializer, SetSerializer
+    NumpyFallbackSerializer, UnitsSerializer, QuantitySerializer,
+    SetSerializer, ProcessSerializer, FunctionSerializer,
+    SequenceDeserializer, DictDeserializer
 )
 
 # import emitters
@@ -55,8 +51,6 @@ from vivarium.core.emitter import (
     Emitter, NullEmitter, RAMEmitter, SharedRamEmitter, DatabaseEmitter
 )
 
-_ = plt  # suppress PyCharm's unused-import warning
-
 
 # register processes
 process_registry.register(DivideCondition.name, DivideCondition)
@@ -97,10 +91,10 @@ divider_registry.register('null', divide_null)
 
 # register serializers
 for SerializerClass in (
-        ProcessSerializer, NumpySerializer, SequenceDeserializer,
-        NumpyBoolSerializer, UnitsSerializer, DictDeserializer,
-        FunctionSerializer, NumpyInt64Serializer, NumpyInt32Serializer,
-        NumpyFloat32Serializer, SetSerializer):
+    NumpyFallbackSerializer, UnitsSerializer, QuantitySerializer,
+    SetSerializer, ProcessSerializer, FunctionSerializer,
+    SequenceDeserializer, DictDeserializer
+    ):
     serializer = SerializerClass()
     serializer_registry.register(
         serializer.name, serializer)
