@@ -59,7 +59,10 @@ def _get_composite_state_recur(
                 process_state = node.initial_state(config.get(node.name))
             elif state_type == 'default':
                 process_state = node.default_state()
-            sub_state = inverse_topology(path, process_state, sub_topology)
+            # Prevent multiupdates from forming when a single process has
+            # multiple ports to the same stores holding a dictionary
+            sub_state = inverse_topology(
+                path, process_state, sub_topology, multi_updates=False)
         else:
             Exception(f'invalid processes {sub_processes} or steps {sub_steps}')
         state = deep_merge(state, sub_state)
