@@ -13,14 +13,15 @@ import uuid
 import warnings
 
 import numpy as np
-from pint import Quantity
+from pint import Unit, Quantity
 from typing import Optional
 
 from vivarium import divider_registry, serializer_registry, updater_registry
 from vivarium.core.process import ParallelProcess, Process
 from vivarium.library.dict_utils import deep_merge, deep_merge_check, MULTI_UPDATE_KEY
-from vivarium.library.topology import without, dict_to_paths
+from vivarium.library.topology import dict_to_paths
 from vivarium.core.types import Processes, Topology, State, Steps, Flow
+from vivarium.core.serialize import QuantitySerializer
 
 _EMPTY_UPDATES = None, None, None, None, None, None
 DEFAULT_SCHEMA = '_default'
@@ -499,6 +500,17 @@ class Store:
         """
         current_schema_value = getattr(self, schema_key)
         if current_schema_value is not None and current_schema_value != new_schema:
+            if schema_key == "units":
+                # Different Python interpreters (inc. from multiprocessing with
+                # spawn start method) yield different hashes for the same value
+                # Reset cached hashes for Pint Units to force rehash before ==
+                if (isinstance(current_schema_value, Unit) 
+                    and isinstance(new_schema, Unit)
+                ):
+                    current_schema_value._units._hash = None
+                    new_schema._units._hash = None
+                    if current_schema_value == new_schema:
+                        return new_schema
             raise ValueError(
                 f"Incompatible schema assignment at {self.path_for()}. "
                 f"Trying to assign the value {new_schema} to key {schema_key}, "
@@ -582,47 +594,44 @@ class Store:
             config = {}  # config needs to be a dict
 
         # remove _output special key. This is used only by schema_topology.
-        config = without(config, '_output')
+        config = config.copy()
+        config.pop('_output', None)
 
         if '*' in config:
-            self._apply_subschema_config(config['*'])
-            config = without(config, '*')
+            self._apply_subschema_config(config.pop('*'))
 
         if '_subschema' in config:
+            subschema = config.pop('_subschema')
             if source:
-                self.sources[source] = config['_subschema']
-            self._apply_subschema_config(config['_subschema'])
-            config = without(config, '_subschema')
+                self.sources[source] = subschema
+            self._apply_subschema_config(subschema)
+            
+            
 
         if '_subtopology' in config:
-            self._merge_subtopology(config['_subtopology'])
-            config = without(config, '_subtopology')
+            self._merge_subtopology(config.pop('_subtopology'))
 
         if source:
             self.sources[source] = config
 
         if '_topology' in config:
-            self.topology = config['_topology']
-            config = without(config, '_topology')
+            self.topology = config.pop('_topology')
 
         if '_flow' in config:
-            flow = config['_flow']
-            config = without(config, '_flow')
+            flow = config.pop('_flow')
             if flow != {}:
                 self.flow = flow
 
         if '_divider' in config:
-            new_divider = config['_divider']
+            new_divider = config.pop('_divider')
             self.divider = self._check_schema_support_defaults(
                 'divider', new_divider, divider_registry)
-            config = without(config, '_divider')
 
         # If emit is set on a branch node, set the entire branch to the
         # emit value.
         if '_emit' in config and self.inner:
-            emit_value = config['_emit']
+            emit_value = config.pop('_emit')
             self.set_emit_value(emit=emit_value)
-            config = without(config, '_emit')
 
         if self.schema_keys & set(config.keys()):
             # We are at a leaf node, so apply its config.
@@ -635,7 +644,8 @@ class Store:
             if '_units' in config:
                 self.units = self._check_schema(
                     'units', config.get('_units'))
-                self.serializer = serializer_registry.access('units')
+                self.serializer = serializer_registry.access(
+                    str(QuantitySerializer.python_type))
 
             if '_serializer' in config:
                 serializer = config['_serializer']
@@ -650,13 +660,15 @@ class Store:
                 if isinstance(self.default, Quantity):
                     self.units = self.units or self.default.units
                     self.serializer = (self.serializer or
-                                       serializer_registry.access('units'))
+                                       serializer_registry.access(
+                                        str(QuantitySerializer.python_type)))
                 elif isinstance(self.default, list) and \
                         len(self.default) > 0 and \
                         isinstance(self.default[0], Quantity):
                     self.units = self.units or self.default[0].units
                     self.serializer = (self.serializer or
-                                       serializer_registry.access('units'))
+                                       serializer_registry.access(
+                                        str(QuantitySerializer.python_type)))
 
             if '_value' in config:
                 self.value = self._check_schema(
@@ -1761,11 +1773,11 @@ class Store:
 
         node = self
         if '_path' in path:
+            path = path.copy()
             node = self._establish_path(
-                path['_path'],
+                path.pop('_path'),
                 {},
                 source=source)
-            path = without(path, '_path')
 
         return node, path
 
