@@ -23,9 +23,13 @@ False
 from typing import Any
 
 import pint
+
 # noinspection PyProtectedMember
-from pint.quantity import _Quantity as Quantity
-from pint.unit import Unit
+from pint import Unit
+try:
+    from pint.quantity import _Quantity as Quantity
+except ImportError:
+    from pint import Quantity
 
 #: Units registry that stores the units used throughout Vivarium
 units = pint.UnitRegistry()
