@@ -14,6 +14,9 @@ class Clock(Process):
     defaults = {
         'time_step': 1.0}
 
+    def initial_state(self, config=None):
+        return {}
+
     def ports_schema(self):
         return {
             'global_time': {
